@@ -28,7 +28,7 @@ import sys
 import tarfile
 import tempfile
 import zipfile
-from contextlib import contextmanager
+from contextlib import contextmanager, closing
 
 try:
     import ConfigParser
@@ -37,7 +37,7 @@ except ImportError:
     import configparser as ConfigParser
 
 
-__version__ = '0.17'
+__version__ = '0.18'
 __author__ = 'Marius Gedminas <marius@gedmin.as>'
 __licence__ = 'MIT'
 __url__ = 'https://github.com/mgedmin/check-manifest'
@@ -133,8 +133,11 @@ def run(command):
 
     Raises CommandFailed in cases of error.
     """
-    pipe = subprocess.Popen(command, stdout=subprocess.PIPE,
-                            stderr=subprocess.STDOUT)
+    try:
+        pipe = subprocess.Popen(command, stdout=subprocess.PIPE,
+                                stderr=subprocess.STDOUT)
+    except OSError as e:
+        raise Failure("could not run %s: %s" % (command, e))
     output = pipe.communicate()[0].decode(locale.getpreferredencoding(False))
     status = pipe.wait()
     if status != 0:
@@ -207,10 +210,10 @@ def get_archive_file_list(archive_filename):
     Supports .tar.gz and .zip.
     """
     if archive_filename.endswith('.zip'):
-        with zipfile.ZipFile(archive_filename) as zf:
+        with closing(zipfile.ZipFile(archive_filename)) as zf:
             return add_directories(zf.namelist())
     elif archive_filename.endswith(('.tar.gz', '.tar.bz2', '.tar')):
-        with tarfile.open(archive_filename) as tf:
+        with closing(tarfile.open(archive_filename)) as tf:
             return tf.getnames()
     else:
         ext = os.path.splitext(archive_filename)[-1]
@@ -367,6 +370,7 @@ SUGGESTIONS = [(re.compile(pattern), suggestion) for pattern, suggestion in [
     # regexp -> suggestion
     ('^([^/]+[.](cfg|ini))$',       r'include \1'),
     ('^([.]travis[.]yml)$',         r'include \1'),
+    ('^([.]coveragerc)$',           r'include \1'),
     ('^([A-Z]+)$',                  r'include \1'),
     ('^(Makefile)$',                r'include \1'),
     ('^[^/]+[.](txt|rst|py)$',      r'include *.\1'),
@@ -558,6 +562,11 @@ def check_manifest(source_tree='.', create=False, update=False,
         with mkdtemp('-sources') as tempsourcedir:
             copy_files(source_files, tempsourcedir)
             if os.path.exists('MANIFEST.in') and 'MANIFEST.in' not in source_files:
+                # See https://github.com/mgedmin/check-manifest/issues/7
+                # if do this, we will emit a warning about MANIFEST.in not
+                # being in source control, if we don't do this, the user
+                # gets confused about their new manifest rules being
+                # ignored.
                 copy_files(['MANIFEST.in'], tempsourcedir)
             info_begin("building an sdist")
             with cd(tempsourcedir):
