@@ -2,9 +2,23 @@ import doctest
 import os
 import shutil
 import subprocess
+import sys
 import tempfile
 import textwrap
-import unittest
+import zipfile
+from contextlib import closing
+
+try:
+    import unittest2 as unittest    # Python 2.6
+except ImportError:
+    import unittest
+
+try:
+    from cStringIO import StringIO  # Python 2.x
+except ImportError:
+    from io import StringIO         # Python 3.x
+
+import mock
 
 
 class Tests(unittest.TestCase):
@@ -19,6 +33,91 @@ class Tests(unittest.TestCase):
         import check_manifest
         check_manifest.warning = self._real_warning
 
+    def make_temp_dir(self):
+        tmpdir = tempfile.mkdtemp(prefix='test-', suffix='-check-manifest')
+        self.addCleanup(shutil.rmtree, tmpdir)
+        return tmpdir
+
+    def create_zip_file(self, filename, filenames):
+        with closing(zipfile.ZipFile(filename, 'w')) as zf:
+            for fn in filenames:
+                zf.writestr(fn, '')
+
+    def test_run_success(self):
+        from check_manifest import run
+        self.assertEqual(run(["true"]), "")
+
+    def test_run_failure(self):
+        from check_manifest import run, CommandFailed
+        with self.assertRaises(CommandFailed) as cm:
+            run(["false"])
+        self.assertEqual(str(cm.exception),
+                         "['false'] failed (status 1):\n")
+
+    def test_run_no_such_program(self):
+        from check_manifest import run, Failure
+        with self.assertRaises(Failure) as cm:
+            run(["there-is-really-no-such-program"])
+        self.assertTrue(
+            str(cm.exception).startswith(
+                "could not run ['there-is-really-no-such-program']:"
+                " [Errno 2] No such file or directory"))
+
+    def test_copy_files(self):
+        from check_manifest import copy_files
+        actions = []
+        with mock.patch('os.path.isdir', lambda d: d in ('b', '/dest/dir')):
+            with mock.patch('os.makedirs',
+                            lambda d: actions.append('makedirs %s' % d)):
+                with mock.patch('os.mkdir',
+                                lambda d: actions.append('mkdir %s' % d)):
+                    with mock.patch('shutil.copy2',
+                                    lambda s, d: actions.append('cp %s %s' % (s, d))):
+                        copy_files(['a', 'b', 'c/d/e'], '/dest/dir')
+        self.assertEqual(
+            actions,
+            [
+                'cp a /dest/dir/a',
+                'mkdir /dest/dir/b',
+                'makedirs /dest/dir/c/d',
+                'cp c/d/e /dest/dir/c/d/e',
+            ])
+
+    def test_get_one_file_in(self):
+        from check_manifest import get_one_file_in
+        with mock.patch('os.listdir', lambda dir: ['a']):
+            self.assertEqual(get_one_file_in('/some/dir'),
+                             '/some/dir/a')
+
+    def test_get_one_file_in_empty_directory(self):
+        from check_manifest import get_one_file_in, Failure
+        with mock.patch('os.listdir', lambda dir: []):
+            with self.assertRaises(Failure) as cm:
+                get_one_file_in('/some/dir')
+            self.assertEqual(str(cm.exception),
+                             "No files found in /some/dir")
+
+    def test_get_one_file_in_too_many(self):
+        from check_manifest import get_one_file_in, Failure
+        with mock.patch('os.listdir', lambda dir: ['b', 'a']):
+            with self.assertRaises(Failure) as cm:
+                get_one_file_in('/some/dir')
+            self.assertEqual(str(cm.exception),
+                             "More than one file exists in /some/dir:\na\nb")
+
+    def test_get_archive_file_list_unrecognized_archive(self):
+        from check_manifest import get_archive_file_list, Failure
+        with self.assertRaises(Failure) as cm:
+            get_archive_file_list('archive.rar')
+        self.assertEqual(str(cm.exception), 'Unrecognized archive type: .rar')
+
+    def test_get_archive_file_list_zip(self):
+        from check_manifest import get_archive_file_list
+        filename = os.path.join(self.make_temp_dir(), 'archive.zip')
+        self.create_zip_file(filename, ['a', 'b/c'])
+        self.assertEqual(get_archive_file_list(filename),
+                         ['a', 'b', 'b/c'])
+
     def test_format_list(self):
         from check_manifest import format_list
         self.assertEqual(format_list([]), "")
@@ -45,6 +144,15 @@ class Tests(unittest.TestCase):
         from check_manifest import strip_toplevel_name, Failure
         self.assertRaises(Failure, strip_toplevel_name, ["a/b", "c/d"])
 
+    def test_detect_vcs_no_vcs(self):
+        from check_manifest import detect_vcs, Failure
+        with mock.patch('check_manifest.VCS.detect', staticmethod(lambda *a: False)):
+            with self.assertRaises(Failure) as cm:
+                detect_vcs()
+            self.assertEqual(str(cm.exception),
+                             "Couldn't find version control data"
+                             " (git/hg/bzr/svn supported)")
+
     def test_normalize_names(self):
         from check_manifest import normalize_names
         self.assertEqual(normalize_names(["a", "b/", "c/d", "e/f/", "g/h/../i"]),
@@ -254,17 +362,17 @@ class Tests(unittest.TestCase):
         self.assertEqual(parse('prune dir/'),
                          (['dir', 'dir/*'], []))
         text = """
-        #exclude *.01
-        exclude *.02
-        exclude *.03 04.*   bar.txt
-        exclude          *.05
-        exclude some/directory/*.cfg
-        global-exclude *.10 *.11
-        global-exclude *.12
-        include *.20
-        prune 30
-        recursive-exclude    40      *.41
-        recursive-exclude 42 *.43 44.*
+            #exclude *.01
+            exclude *.02
+            exclude *.03 04.*   bar.txt
+            exclude          *.05
+            exclude some/directory/*.cfg
+            global-exclude *.10 *.11
+            global-exclude *.12
+            include *.20
+            prune 30
+            recursive-exclude    40      *.41
+            recursive-exclude 42 *.43 44.*
         """
         # Keep the indentation visually clear in the test, but remove
         # leading whitespace programmatically.
@@ -272,24 +380,23 @@ class Tests(unittest.TestCase):
         self.assertEqual(
             parse(text),
             ([
-             'bar.txt',
-             '*.10',
-             '*.11',
-             '*.12',
-             '30',
-             '30/*',
-             '40/*.41',
-             '42/*.43',
-             '42/44.*',
-             '42/*/44.*',
-             ],
-             [
-             '[^/]*\.02',
-             '[^/]*\.03',
-             '04\.[^/]*',
-             '[^/]*\.05',
-             'some/directory/[^/]*\.cfg',
-             ]))
+                'bar.txt',
+                '*.10',
+                '*.11',
+                '*.12',
+                '30',
+                '30/*',
+                '40/*.41',
+                '42/*.43',
+                '42/44.*',
+                '42/*/44.*',
+            ], [
+                '[^/]*\.02',
+                '[^/]*\.03',
+                '04\.[^/]*',
+                '[^/]*\.05',
+                'some/directory/[^/]*\.cfg',
+            ]))
 
 
 class VCSMixin(object):
@@ -304,12 +411,14 @@ class VCSMixin(object):
         shutil.rmtree(self.tmpdir)
 
     def _run(self, *command):
-        try:
-            subprocess.check_output(command, stderr=subprocess.STDOUT)
-        except subprocess.CalledProcessError as e:
+        p = subprocess.Popen(command, stdout=subprocess.PIPE,
+                             stderr=subprocess.STDOUT)
+        stdout, stderr = p.communicate()
+        rc = p.wait()
+        if rc:
             print(' '.join(command))
-            print(e.output)
-            raise
+            print(stdout)
+            raise subprocess.CalledProcessError(rc, command[0], output=stdout)
 
     def _create_file(self, filename):
         assert not os.path.isabs(filename)
@@ -409,6 +518,66 @@ class TestSvn(VCSMixin, unittest.TestCase):
         self._run('svn', 'commit', '-m', 'Initial')
 
 
+class TestUserInterface(unittest.TestCase):
+
+    def setUp(self):
+        self.real_stdout = sys.stdout
+        self.real_stderr = sys.stderr
+        sys.stdout = StringIO()
+        sys.stderr = StringIO()
+
+    def tearDown(self):
+        sys.stderr = self.real_stderr
+        sys.stdout = self.real_stdout
+
+    def test_info(self):
+        from check_manifest import info
+        info("Reticulating splines")
+        self.assertEqual(sys.stdout.getvalue(),
+                         "Reticulating splines\n")
+
+    def test_info_begin_continue_end(self):
+        from check_manifest import info_begin, info_continue, info_end
+        info_begin("Reticulating splines...")
+        info_continue(" nearly done...")
+        info_continue(" almost done...")
+        info_end(" done!")
+        self.assertEqual(
+            sys.stdout.getvalue(),
+            "Reticulating splines... nearly done... almost done... done!\n")
+
+    def test_info_emits_newline_when_needed(self):
+        from check_manifest import info_begin, info
+        info_begin("Computering...")
+        info("Forgot to turn the gas off!")
+        self.assertEqual(
+            sys.stdout.getvalue(),
+            "Computering...\n"
+            "Forgot to turn the gas off!\n")
+
+    def test_warning(self):
+        from check_manifest import info_begin, warning
+        info_begin("Computering...")
+        warning("Forgot to turn the gas off!")
+        self.assertEqual(
+            sys.stdout.getvalue(),
+            "Computering...\n")
+        self.assertEqual(
+            sys.stderr.getvalue(),
+            "Forgot to turn the gas off!\n")
+
+    def test_error(self):
+        from check_manifest import info_begin, error
+        info_begin("Computering...")
+        error("Forgot to turn the gas off!")
+        self.assertEqual(
+            sys.stdout.getvalue(),
+            "Computering...\n")
+        self.assertEqual(
+            sys.stderr.getvalue(),
+            "Forgot to turn the gas off!\n")
+
+
 def test_suite():
     return unittest.TestSuite([
         unittest.makeSuite(Tests),
@@ -416,5 +585,6 @@ def test_suite():
         unittest.makeSuite(TestBzr),
         unittest.makeSuite(TestHg),
         unittest.makeSuite(TestSvn),
+        unittest.makeSuite(TestUserInterface),
         doctest.DocTestSuite('check_manifest'),
     ])
